'''
ADT7410 temperature sensor
MicroPython driver for BBC micro:bit

This driver concentrates on returning
temperature measurements.
No watchdog functions are provided.

REFERENCES:
ADT7410 Datasheet
https://www.fredscave.com/drivers/024-adt7410.html

AUTHOR: fredscave.com
DATE  : 2025/04
VERSION : 1.00
'''

from microbit import i2c, sleep
from micropython import const

I2C_ADDR    = const(0x48)
CMD_RESET   = const(0x2F)

# Registers
REG_TEMP    = const(0x00)
REG_CONFIG  = const(0x03)

# Modes
CONTINUOUS    = const(0x00)
ONE_SHOT      = const(0x20)
ONE_SPS       = const(0x40)
SHUT_DOWN     = const(0x60)

# Resolution
RES_HI = const(0x80)
RES_LO = const(0x00)

# Centigrade to Fahrenheit
CtoF = lambda C, d=1: round((C * 9/5) +32, d)

class ADT7410():
    # Constructor sets up I2C address, resolution and mode
    def __init__(self, Addr = I2C_ADDR,
                 Resolution = RES_LO, Mode = CONTINUOUS):
        self.buf = bytearray(2)
        self.Addr = Addr
        self._Reset()
        if Resolution == RES_HI:
            self.Resolution = RES_HI
        else:
            self.Resolution = RES_LO
        if Mode not in (CONTINUOUS, ONE_SHOT, ONE_SPS):
            self.Mode = CONTINUOUS
        else:
            self.Mode = Mode
        self._SetMode()
        self.UserShutdown = False

    # User method to call to return latest temperature.
    # Resolution and sampling mode are as set by the
    # constructor.
    def Read(self):
        if self.Mode == ONE_SHOT and not self.UserShutdown:
            self.WakeUp()
        i2c.write(self.Addr, bytes([REG_TEMP]))
        self.buf = i2c.read(self.Addr, 2)
        return self._Temperature()

    # Forces the sensor into shutdown mode.
    # All chip internals are switched off.
    # The I2C interface is still active.
    # The last temperature conversion is still available.
    def ShutDown(self):
        i2c.write(self.Addr, bytes([REG_CONFIG, SHUT_DOWN]))
        self.UserShutdown = True

    # Wake upt he sensor from shutdown mode.
    # Temperature conversions commence based on
    # the mode and resolution set by the constructor.
    def WakeUp(self):
        self._SetMode()
        self.UserShutdown = False

    # Soft reset of the chip. All default values
    # are loaded which override any settings that
    # that may have been made in the constructor.
    def _Reset(self):
        i2c.write(I2C_ADDR, bytes([CMD_RESET]))
        sleep(300)

    # Writes a byte to the Configuration register
    # to set the resolution and mode as passed to
    # the constructor.
    def _SetMode(self):
        Config = self.Mode
        if self.Resolution == RES_HI:
            Config = Config | RES_HI
        i2c.write(self.Addr, bytes([REG_CONFIG, Config]))
        sleep(250)

    # Converts the 13-bit or 16-bit two's complement
    # binary value to the decimal value.
    def _Temperature(self):
        positive = (self.buf[0] >> 7) == 0
        if self.Resolution == RES_LO:
            raw = self.buf[0]*256 + self.buf[1] >> 3
            if positive:
               return raw/16
            else:
               return (raw - 8192) / 16
        else:
            raw = self.buf[0]*256 + self.buf[1]
            if positive:
                return raw/128
            else:
                return (raw - 65536) / 128









